@@ -46,10 +46,11 @@ typedef struct {
 } MonitorHandleContext;
 
 struct _EogListStorePrivate {
-	GList *monitors;       /* Monitors for the directories */
-	gint initial_image;    /* The image that should be selected firstly by the view. */
-	GdkPixbuf *busy_image; /* Hourglass image */
-	GMutex *mutex;         /* Mutex for saving the jobs in the model */
+	GList *monitors;          /* Monitors for the directories */
+	gint initial_image;       /* The image that should be selected firstly by the view. */
+	GdkPixbuf *busy_image;    /* Loading image icon */
+	GdkPixbuf *missing_image; /* Missing image icon */
+	GMutex *mutex;            /* Mutex for saving the jobs in the model */
 };
 
 static void
@@ -92,6 +93,11 @@ eog_list_store_dispose (GObject *object)
 		store->priv->busy_image = NULL;
 	}
 
+	if(store->priv->missing_image != NULL) {
+		g_object_unref (store->priv->missing_image);
+		store->priv->missing_image = NULL;
+	}
+
 	g_mutex_free (store->priv->mutex);
 
 	G_OBJECT_CLASS (eog_list_store_parent_class)->dispose (object);
@@ -140,21 +146,18 @@ eog_list_store_compare_func (GtkTreeModel *model,
 }
 
 static GdkPixbuf *
-eog_list_store_get_loading_icon (void)
+eog_list_store_get_icon (const gchar *icon_name)
 {
 	GError *error = NULL;
 	GtkIconTheme *icon_theme;
 	GdkPixbuf *pixbuf;
 	
-	icon_theme = gtk_icon_theme_get_default();
+	icon_theme = gtk_icon_theme_get_default ();
 
-	/* FIXME: The 16 added to EOG_LIST_STORE_THUMB_SIZE should be 
-	   calculated from the BLUR_RADIUS and RECTANGLE_OUTLINE macros 
-	   in eog-thumb-shadow.c */
 	pixbuf = gtk_icon_theme_load_icon (icon_theme,
-					   "image-loading", /* icon name */
-					   EOG_LIST_STORE_THUMB_SIZE + 16, /* size */
-					   0,  /* flags */
+					   icon_name, 
+					   EOG_LIST_STORE_THUMB_SIZE, 
+					   0,
 					   &error);
 
 	if (!pixbuf) {
@@ -183,7 +186,8 @@ eog_list_store_init (EogListStore *self)
 	self->priv->monitors = NULL;
 	self->priv->initial_image = -1;
 
-	self->priv->busy_image = eog_list_store_get_loading_icon ();
+	self->priv->busy_image = eog_list_store_get_icon ("image-loading");
+	self->priv->missing_image = eog_list_store_get_icon ("image-missing");
 
 	self->priv->mutex = g_mutex_new ();
 
@@ -247,12 +251,28 @@ is_file_in_list_store (EogListStore *store,
 	return found;
 }
 
+static gboolean
+is_file_in_list_store_uri (EogListStore *store,
+			   const GnomeVFSURI *uri,
+			   GtkTreeIter *iter_found)
+{
+	gchar *uri_str;
+	gboolean result;
+	
+	uri_str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+
+	result = is_file_in_list_store (store, uri_str, iter_found);
+
+	g_free (uri_str);
+
+	return result;
+}
+
 static void
 eog_job_thumbnail_cb (EogJobThumbnail *job, gpointer data)
 {
 	EogListStore *store;
 	GtkTreeIter iter;
-	gchar *filename;
 	EogImage *image;
 	GdkPixbuf *thumbnail;
 	
@@ -260,26 +280,29 @@ eog_job_thumbnail_cb (EogJobThumbnail *job, gpointer data)
 
 	store = EOG_LIST_STORE (data);
 
-	filename = gnome_vfs_uri_to_string (job->uri_entry, GNOME_VFS_URI_HIDE_NONE);
-
-	if (is_file_in_list_store (store, filename, &iter)) {
+	if (is_file_in_list_store_uri (store, job->uri_entry, &iter)) {
 		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 
 				    EOG_LIST_STORE_EOG_IMAGE, &image,
 				    -1);
 
-		eog_image_set_thumbnail (image, job->thumbnail);
-		/* getting the thumbnail, in case it needed transformations */
-		thumbnail = eog_image_get_thumbnail (image);
+		if (job->thumbnail) {
+			eog_image_set_thumbnail (image, job->thumbnail);
+
+			/* Getting the thumbnail, in case it needed 
+ 			 * transformations */
+			thumbnail = eog_image_get_thumbnail (image);
+		} else {
+			thumbnail = g_object_ref (store->priv->missing_image);
+		}
 
 		gtk_list_store_set (GTK_LIST_STORE (store), &iter, 
 				    EOG_LIST_STORE_THUMBNAIL, thumbnail,
 				    EOG_LIST_STORE_THUMB_SET, TRUE,
 				    EOG_LIST_STORE_EOG_JOB, NULL,
 				    -1);
+
 		g_object_unref (thumbnail);
 	}
-
-	g_free (filename);
 }
 
 static void
@@ -329,8 +352,10 @@ eog_list_store_append_image (EogListStore *store, EogImage *image)
 			    -1);
 }
 
-void 
-eog_list_store_append_image_from_uri (EogListStore *store, GnomeVFSURI *uri_entry)
+static void 
+eog_list_store_append_image_from_uri (EogListStore *store, 
+				      GnomeVFSURI *uri_entry,
+				      gboolean is_monitored)
 {
 	EogImage *image;
 	
@@ -338,6 +363,8 @@ eog_list_store_append_image_from_uri (EogListStore *store, GnomeVFSURI *uri_entr
 
 	image = eog_image_new_uri (uri_entry);
 
+	eog_image_set_is_monitored (image, is_monitored);
+
 	eog_list_store_append_image (store, image);
 }
 
@@ -366,7 +393,7 @@ vfs_monitor_dir_cb (GnomeVFSMonitorHandle *handle,
 		} else {
 			if (eog_image_is_supported_mime_type (mimetype)) {
 				uri = gnome_vfs_uri_new (info_uri);
-				eog_list_store_append_image_from_uri (store, uri);
+				eog_list_store_append_image_from_uri (store, uri, TRUE);
 				gnome_vfs_uri_unref (uri);
 			}
 		}
@@ -376,15 +403,29 @@ vfs_monitor_dir_cb (GnomeVFSMonitorHandle *handle,
 
 	case GNOME_VFS_MONITOR_EVENT_DELETED:
 		if (is_file_in_list_store (store, info_uri, &iter)) {
-			eog_list_store_remove (store, &iter);
+			EogImage *image;
+
+			gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
+					    EOG_LIST_STORE_EOG_IMAGE, &image,
+					    -1);
+
+			if (eog_image_get_status (image) != EOG_IMAGE_STATUS_SAVING) {
+				eog_list_store_remove (store, &iter);
+			} else {
+				eog_image_restore_status (image);
+			}
 		}
 		break;
 
 	case GNOME_VFS_MONITOR_EVENT_CREATED:
-		if (is_file_in_list_store (store, info_uri, NULL)) {
-			uri = gnome_vfs_uri_new (info_uri);
-			eog_list_store_append_image_from_uri (store, uri);
-			gnome_vfs_uri_unref (uri);
+		if (!is_file_in_list_store (store, info_uri, NULL)) {
+			mimetype = gnome_vfs_get_mime_type (info_uri);
+			if (eog_image_is_supported_mime_type (mimetype)) {
+				uri = gnome_vfs_uri_new (info_uri);
+				eog_list_store_append_image_from_uri (store, uri, TRUE);
+				gnome_vfs_uri_unref (uri);
+			}
+			g_free (mimetype);
 		}
 		break;
 
@@ -432,7 +473,7 @@ directory_visit_cb (const gchar *rel_uri,
 
 	if (load_uri) {
 		uri = gnome_vfs_uri_append_file_name (ctx->uri, rel_uri);
-		eog_list_store_append_image_from_uri (store, uri);
+		eog_list_store_append_image_from_uri (store, uri, TRUE);
 	}
 
 	return TRUE;
@@ -531,18 +572,17 @@ eog_list_store_add_uris (EogListStore *store, GList *uri_list)
 			if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
 				eog_list_store_append_directory (store, uri, info);
 
-				if (!is_file_in_list_store (store, 
-							    gnome_vfs_uri_to_string (initial_uri, 
-										     GNOME_VFS_URI_HIDE_NONE), 
-							    &iter)) {
-					eog_list_store_append_image_from_uri (store, initial_uri);
+				if (!is_file_in_list_store_uri (store,
+								initial_uri, 
+								&iter)) {
+					eog_list_store_append_image_from_uri (store, initial_uri, TRUE);
 				}
 			} else {
-				eog_list_store_append_image_from_uri (store, initial_uri);
+				eog_list_store_append_image_from_uri (store, initial_uri, FALSE);
 			}
 		} else if (info->type == GNOME_VFS_FILE_TYPE_REGULAR && 
 			   g_list_length (uri_list) > 1) {
-			eog_list_store_append_image_from_uri (store, uri);
+			eog_list_store_append_image_from_uri (store, uri, FALSE);
 		}
 	}
 
@@ -553,9 +593,7 @@ eog_list_store_add_uris (EogListStore *store, GList *uri_list)
 					      GTK_SORT_ASCENDING);
 	
 	if (initial_uri && 
-	    is_file_in_list_store (store, 
-				   gnome_vfs_uri_to_string (initial_uri, GNOME_VFS_URI_HIDE_NONE), 
-				   &iter)) {
+	    is_file_in_list_store_uri (store, initial_uri, &iter)) {
 		store->priv->initial_image = eog_list_store_get_pos_by_iter (store, &iter);
 		gnome_vfs_uri_unref (initial_uri);
 	} else {
@@ -567,21 +605,17 @@ void
 eog_list_store_remove_image (EogListStore *store, EogImage *image)
 {
 	GtkTreeIter iter;
-	gchar *file;
 	GnomeVFSURI *uri;
 
 	g_return_if_fail (EOG_IS_LIST_STORE (store));
 	g_return_if_fail (EOG_IS_IMAGE (image));
 
 	uri = eog_image_get_uri (image);
-	file = gnome_vfs_uri_to_string (uri, 
-					GNOME_VFS_URI_HIDE_NONE);
-	gnome_vfs_uri_unref (uri);
-	
-	if (is_file_in_list_store (store, file, &iter)) {
+
+	if (is_file_in_list_store_uri (store, uri, &iter)) {
 		eog_list_store_remove (store, &iter);
 	}
-	g_free (file);
+	gnome_vfs_uri_unref (uri);
 }
 
 GtkListStore *
@@ -602,7 +636,6 @@ eog_list_store_new_from_glist (GList *list)
 gint
 eog_list_store_get_pos_by_image (EogListStore *store, EogImage *image)
 {
-	gchar *file;
 	GtkTreeIter iter;
 	gint pos = -1;
 	GnomeVFSURI *uri;
@@ -611,15 +644,12 @@ eog_list_store_get_pos_by_image (EogListStore *store, EogImage *image)
 	g_return_val_if_fail (EOG_IS_IMAGE (image), -1);
 	
 	uri = eog_image_get_uri (image);
-	file = gnome_vfs_uri_to_string (uri, 
-					GNOME_VFS_URI_HIDE_NONE);
-	gnome_vfs_uri_unref (uri);
 
-	if (is_file_in_list_store (store, file, &iter)) {
+	if (is_file_in_list_store_uri (store, uri, &iter)) {
 		pos = eog_list_store_get_pos_by_iter (store, &iter);
 	}
 
-	g_free (file);
+	gnome_vfs_uri_unref (uri);
 	return pos;
 }
 
@@ -703,8 +733,14 @@ eog_list_store_add_thumbnail_job (EogListStore *store, GtkTreeIter *iter)
 	
 	gtk_tree_model_get (GTK_TREE_MODEL (store), iter, 
 			    EOG_LIST_STORE_EOG_IMAGE, &image,
+			    EOG_LIST_STORE_EOG_JOB, &job,
 			    -1);
-	
+
+	if (job != NULL) {
+		g_object_unref (image);
+		return;
+	}
+
 	uri = eog_image_get_uri (image);
 	job = eog_job_thumbnail_new (uri);
 	gnome_vfs_uri_unref (uri);
@@ -728,7 +764,7 @@ void
 eog_list_store_thumbnail_set (EogListStore *store, 
 			      GtkTreeIter *iter)
 {
-	gboolean *thumb_set;
+	gboolean thumb_set = FALSE;
 
 	gtk_tree_model_get (GTK_TREE_MODEL (store), iter, 
 			    EOG_LIST_STORE_THUMB_SET, &thumb_set,
