@@ -1,6 +1,6 @@
 /* Eye Of Gnome - Image Store
  *
- * Copyright (C) 2006 The Free Software Foundation
+ * Copyright (C) 2006-2007 The Free Software Foundation
  *
  * Author: Claudio Saavedra <csaavedra@alumnos.utalca.cl>
  *
@@ -80,9 +80,6 @@ GtkListStore   *eog_list_store_new_from_glist 	     (GList *list);
 void            eog_list_store_append_image 	     (EogListStore *store, 
 						      EogImage     *image);
 
-void            eog_list_store_append_image_from_uri (EogListStore *store, 
-						      GnomeVFSURI  *uri_entry);
-
 void            eog_list_store_add_uris 	     (EogListStore *store, 
 						      GList        *uri_list);
 
@@ -108,6 +105,9 @@ void            eog_list_store_thumbnail_set         (EogListStore *store,
 void            eog_list_store_thumbnail_unset       (EogListStore *store, 
 						      GtkTreeIter *iter);
 
+void            eog_list_store_thumbnail_refresh     (EogListStore *store,
+						      GtkTreeIter *iter);
+
 G_END_DECLS
 
 #endif
