@@ -292,10 +292,12 @@ tb_on_button_press_event_cb (GtkWidget *tb, GdkEventButton *event,
 		    return FALSE;
 	    }
 
-	    gtk_icon_view_unselect_all (GTK_ICON_VIEW (tb));
-	    gtk_icon_view_select_path (GTK_ICON_VIEW (tb), path);
-	    gtk_icon_view_set_cursor (GTK_ICON_VIEW (tb), path, NULL, FALSE);
-
+	    if (!gtk_icon_view_path_is_selected (GTK_ICON_VIEW (tb), path) ||
+		eog_thumb_view_get_n_selected (EOG_THUMB_VIEW (tb)) != 1) {
+		gtk_icon_view_unselect_all (GTK_ICON_VIEW (tb));
+		gtk_icon_view_select_path (GTK_ICON_VIEW (tb), path);
+		gtk_icon_view_set_cursor (GTK_ICON_VIEW (tb), path, NULL, FALSE);
+	    }
 	    eog_thumb_view_popup_menu (EOG_THUMB_VIEW (tb), event);
 	    
 	    gtk_tree_path_free (path);
@@ -443,9 +445,12 @@ tb_on_query_tooltip_cb (GtkWidget  *widget,
 #ifdef HAVE_EXIF
 	if (exif_data) {
 		gchar *extra_info, *tmp, *date;
+		/* The EXIF standard says that the DATE_TIME tag is 
+		 * 20 bytes long. A 32-byte buffer should be large enough. */
+		gchar time_buffer[32];
 		
 		date = eog_exif_util_format_date (
-				eog_exif_util_get_value (exif_data, EXIF_TAG_DATE_TIME));
+				eog_exif_util_get_value (exif_data, EXIF_TAG_DATE_TIME, time_buffer, 32));
 
 		extra_info = g_strdup_printf ("\n%s %s", _("Taken on"), date);
 
@@ -616,11 +621,7 @@ eog_thumb_view_get_first_selected_image (EogThumbView *tb)
 
 	GtkTreePath *path = (GtkTreePath *) (list->data);
 
-	/* debugging purposes */
-	gchar *text_path;
 	image = eog_thumb_view_get_image_from_path (tb, path);
-	text_path = gtk_tree_path_to_string (path);
-	g_free (text_path);
 
 	g_list_foreach (list, (GFunc) gtk_tree_path_free , NULL);
 	g_list_free (list);
